/*********************************************************************
 * Software License Agreement (BSD License)
 *
 *  Copyright (c) 2008, Willow Garage, Inc.
 *  All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *   * Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *   * Redistributions in binary form must reproduce the above
 *     copyright notice, this list of conditions and the following
 *     disclaimer in the documentation and/or other materials provided
 *     with the distribution.
 *   * Neither the name of the Willow Garage nor the names of its
 *     contributors may be used to endorse or promote products derived
 *     from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 *  FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 *  BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 *  LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 *  CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 *  LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
 *  ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 *  POSSIBILITY OF SUCH DAMAGE.
 *********************************************************************/

#ifndef RIQ_HAND_ETHERCAT_HARDWARE__RIQ_HAND_COMMAND_H
#define RIQ_HAND_ETHERCAT_HARDWARE__RIQ_HAND_COMMAND_H

#include <stdint.h>

namespace riq_hand_ethercat_hardware
{


struct RIQActionRequest
{
  union {
    uint8_t raw_;
    struct {
      uint8_t initialize_    : 1;  //! When set to 0 will reset gripper, setting to 1 will initialize gripper
      uint8_t mode_          : 2;  //! Gripper grasp mode 
      uint8_t grip_          : 2;  //! Open,Close,Stop
      uint8_t parital_open_  : 1;  //! Open all the way, or stop at given position
      uint8_t partial_close_ : 1;  //! Close all the way, or stop at given position
      uint8_t reserved       : 1;
      } __attribute__ ((__packed__));
    } __attribute__ ((__packed__));

  enum {CYLINDRICAL_MODE=0, PINCH_MODE=1, SPHERIOD_MODE=2, SCISSORS_MODE=3};
  enum {GRIP_STOP=0, GRIP_OPEN=1, GRIP_CLOSE=2, GRIP_STOP_X=3};
  enum {OPEN_MAX=0, OPEN_PARTIAL=1};
  enum {CLOSE_MAX=0, CLOSE_PARTIAL=1};

  static const char* modeString(unsigned mode);
  static const char* gripString(unsigned grip);

  const char* modeString() const {return modeString(mode_);}
  const char* gripString() const {return gripString(grip_);}
  
} __attribute__ ((__packed__));  


struct RIQSafetyShutdown
{
  union {
    uint8_t raw_;
    struct {
      uint8_t reserved_    : 4;  
      uint8_t timeout_     : 4;
    } __attribute__ ((__packed__));
  } __attribute__ ((__packed__));

  enum {SHUTDOWN_TIMEOUT_NONE=0, 
        SHUTDOWN_TIMEOUT_20ms=1, 
        SHUTDOWN_TIMEOUT_40ms=2,
        SHUTDOWN_TIMEOUT_80ms=3,
        SHUTDOWN_TIMEOUT_160ms=4,
        SHUTDOWN_TIMEOUT_320ms=5,
        SHUTDOWN_TIMEOUT_640ms=6,
        SHUTDOWN_TIMEOUT_1280ms=7,
        SHUTDOWN_TIMEOUT_2560ms=8,
        SHUTDOWN_TIMEOUT_5120ms=9,
        SHUTDOWN_TIMEOUT_10240ms=10};
} __attribute__ ((__packed__));



struct RIQHandCommandEcat
{
  RIQActionRequest action_request_;
  RIQSafetyShutdown safety_shutdown_; 
  uint8_t velocity_;
  uint8_t force_;
  uint8_t partial_open_;
  uint8_t partial_close_;
  uint8_t empty_[6];

  void zero();
} __attribute__ ((__packed__));



}; //namespace riq_hand_ethercat_hardware


#endif /* RIQ_HAND_ETHERCAT_HARDWARE__RIQ_HAND_COMMAND_H */
