#include "riq_hand_ethercat_hardware/riq_hand_state.h"

namespace riq_hand_ethercat_hardware
{


const char* RIQGripperStatus::modeString(unsigned mode)
{
  switch(mode)
  {
  case CYLINDRICAL_MODE:
    return "Cylindrical";
  case PINCH_MODE:
    return "Pinch";
  case SPHERIOD_MODE:
    return "Spheriod";
  case SCISSORS_MODE:
    return "Scissors";
  default:
    return "Unknown";
  };
}

const char* RIQGripperStatus::gripString(unsigned grip)
{
  switch(grip)
  {
  case  GRIP_STOPPED: 
    return "Stop";
  case GRIP_OPENED:
    return "Open";
  case GRIP_CLOSED: 
    return "Close";
  case GRIP_INITIALIZING_OR_CHANGING:
    return "Mode Change, Initialization";
  default:
    return "Unknown";
  };
}

const char* RIQGripperStatus::statusString(unsigned status)
{
  switch (status)
  {
  case ACTION_FAULTED:
    return "Requested action has faulted";
  case ACTION_IN_PROGRESS: 
    return "Requested action is in progress";
  case ACTION_ILLEGAL_UNDEFINED: 
    return "Illegal/Undefined";
  case ACTION_SUCCESSFUL:
    return "Requested action was successfully completed";
  default:
    return "Unknown";
  }
}


const char* RIQObjectStatus::detectedString(unsigned detected, bool in_scissors_mode)
{
  if (in_scissors_mode)
  {
    switch (detected)
    {
    case OBJECT_NOT_DETECTED:
      return "No object detected";
    case OBJECT_DETECTED_1_FINGER:
    case OBJECT_DETECTED_2_FINGERS:
      return "Invalid in scissors mode";
    case OBJECT_DETECTED_ALL_FINGERS:
      return "Scissors detected an object";
    default:
      return "Unknown";
    }
  } 
  else 
  {
    switch (detected)
    {
    case OBJECT_NOT_DETECTED:
      return "No object detected";
    case OBJECT_DETECTED_1_FINGER:
      return "One finger detected an object";
    case OBJECT_DETECTED_2_FINGERS:
    return "Two fingers detected an object";
    case OBJECT_DETECTED_ALL_FINGERS:
      return "All fingers detected an object";
    default:
      return "Unknown";
    } 
  }
}

const char* RIQObjectStatus::wasDetectedString(bool was_detected)
{
  return (was_detected) ? "An object was detected by the " : "No object was detected by the ";
}



const char* RIQHandStateEcat::faultString(unsigned fault)
{
 switch (fault)
 {
 case FAULT_NEED_INITIALIZATION_TO_COMPLETE:
   return "Action delayed, initialization must be completed prior to action";
 case FAULT_NEED_MODE_CHANGE_TO_COMPLETE:
   return "Action delayed, mode change must be completed priod to action";
 case FAULT_COMMUNICATION_TIMEOUT:
   return "Communication timeout, Gripper is stopped";   
 case FAULT_INSUFFICIENT_SUPPLY_VOLTAGE:
   return "Insufficient supply voltage, Gripper is stoppped";
 case FAULT_CHANGING_MODE_INTERFERENCE_SCISSORS:
   return "Changing mode fault, interferences detected on Scissors";
 case FAULT_OPENING_INTERFERENCE_FINGERS:
   return "Gripper opening fault, interferences detected on Fingers";
 case FAULT_OPENING_INTERFERENCE_SCISSORS:
   return "Gripper opening fault, interferences detected on Scissors";
 case FAULT_CLOSING_INTERFERENCE_FINGERS:
   return "Gripper closing fault, abnormal displacement of Fingers";
 case FAULT_CLOSING_INTERFERENCE_SCISSORS:
   return "Gripper closing fault, abnormal displacement of Scissors";
 case FAULT_MAJOR_SCISSORS_DISPLACEMENT:
   return "Initialization fault, insufficient Scissors displacement";
 case FAULT_MAJOR_FINGER_DISPLACEMENT:
   return "Initialization fault, insufficient Fingers displacement";
 case NO_FAULT:
   return "No fault";
 default:
   return "Unknown fault";
 }
}


double RIQHandStateEcat::convertPosition(unsigned position)
{
  return double(position) * (1.0 / 255.0);
}


double RIQHandStateEcat::convertCurrent(unsigned current)
{
  // LSB equals 0.1 mA
  return double(current) * 0.1 * 0.001;  // value * 0.1 mA/LSB * 0.001 Amps/mAmps
}

}; //namespace riq_hand_ethercat_hardware
